@@ -86,10 +86,11 @@ int makeQuerySocket(const ComboAddress& local, bool udpOrTCP, bool nonLocalBind)
   }
   else {
     // tcp, let the kernel figure out the port
-    // cerr<<"letting kernel pick TCP port"<<endl;
     ourLocal.sin4.sin_port = 0;
-    if(::bind(sock, (struct sockaddr *)&ourLocal, ourLocal.getSocklen()) < 0)
+    if(::bind(sock, (struct sockaddr *)&ourLocal, ourLocal.getSocklen()) < 0) {
+      closesocket(sock);
       throw PDNSException("Resolver binding to local TCP socket on "+ourLocal.toString()+": "+stringerror());
+    }
   }
   return sock;
 }
@@ -99,34 +100,37 @@ Resolver::Resolver()
   locals["default4"] = -1;
   locals["default6"] = -1;
   try {
-    locals["default4"] = makeQuerySocket(ComboAddress(::arg()["query-local-address"]), true, ::arg().mustDo("non-local-bind"));
+    if(!::arg()["query-local-address"].empty())
+      locals["default4"] = makeQuerySocket(ComboAddress(::arg()["query-local-address"]), true, ::arg().mustDo("non-local-bind"));
     if(!::arg()["query-local-address6"].empty())
       locals["default6"] = makeQuerySocket(ComboAddress(::arg()["query-local-address6"]), true, ::arg().mustDo("non-local-bind"));
   }
   catch(...) {
     if(locals["default4"]>=0)
       close(locals["default4"]);
+    if(locals["default6"]>=0)
+      close(locals["default6"]);
     throw;
   }
 }
 
 Resolver::~Resolver()
 {
-  for(std::map<std::string,int>::iterator iter = locals.begin(); iter != locals.end(); iter++) {
-    if (iter->second >= 0)
-      close(iter->second);
+  for (auto& iter: locals) {
+    if (iter.second >= 0)
+      close(iter.second);
   }
 }
 
 uint16_t Resolver::sendResolve(const ComboAddress& remote, const ComboAddress& local,
-                               const DNSName &domain, int type, bool dnssecOK,
+                               const DNSName &domain, int type, int *localsock, bool dnssecOK,
                                const DNSName& tsigkeyname, const DNSName& tsigalgorithm,
                                const string& tsigsecret)
 {
   uint16_t randomid;
   vector<uint8_t> packet;
   DNSPacketWriter pw(packet, domain, type);
-  pw.getHeader()->id = randomid = dns_random(0xffff);
+  pw.getHeader()->id = randomid = dns_random_uint16();
 
   if(dnssecOK) {
     pw.addOpt(2800, 0, EDNSOpts::DNSSECOK);
@@ -153,39 +157,37 @@ uint16_t Resolver::sendResolve(const ComboAddress& remote, const ComboAddress& l
   if (local.sin4.sin_family == 0) {
     // up to us.
     sock = remote.sin4.sin_family == AF_INET ? locals["default4"] : locals["default6"];
+    if (sock == -1) {
+      string ipv = remote.sin4.sin_family == AF_INET ? "4" : "6";
+      string qla = remote.sin4.sin_family == AF_INET ? "" : "6";
+      throw ResolverException("No IPv" + ipv + " socket available, is query-local-address" + qla + " unset?");
+    }
   } else {
     std::string lstr = local.toString();
     std::map<std::string, int>::iterator lptr;
-    // see if there is a local
 
+    // reuse an existing local socket or make a new one
     if ((lptr = locals.find(lstr)) != locals.end()) {
       sock = lptr->second;
     } else {
       // try to make socket
       sock = makeQuerySocket(local, true);
       if (sock < 0)
-        throw ResolverException("Unable to create socket to "+remote.toStringWithPort()+": "+stringerror());
+        throw ResolverException("Unable to create local socket on "+lstr+" to "+remote.toStringWithPort()+": "+stringerror());
       setNonBlocking( sock );
       locals[lstr] = sock;
     }
   }
 
+  if (localsock != nullptr) {
+    *localsock = sock;
+  }
   if(sendto(sock, &packet[0], packet.size(), 0, (struct sockaddr*)(&remote), remote.getSocklen()) < 0) {
     throw ResolverException("Unable to ask query of "+remote.toStringWithPort()+": "+stringerror());
   }
   return randomid;
 }
 
-uint16_t Resolver::sendResolve(const ComboAddress& remote, const DNSName &domain,
-                               int type, bool dnssecOK,
-                               const DNSName& tsigkeyname, const DNSName& tsigalgorithm,
-                               const string& tsigsecret)
-{
-  ComboAddress local;
-  local.sin4.sin_family = 0;
-  return this->sendResolve(remote, local, domain, type, dnssecOK, tsigkeyname, tsigalgorithm, tsigsecret);
-}
-
 static int parseResult(MOADNSParser& mdp, const DNSName& origQname, uint16_t origQtype, uint16_t id, Resolver::res_t* result)
 {
   result->clear();
@@ -199,31 +201,34 @@ static int parseResult(MOADNSParser& mdp, const DNSName& origQname, uint16_t ori
     if(mdp.d_header.qdcount != 1)
       throw ResolverException("resolver: received answer with wrong number of questions ("+itoa(mdp.d_header.qdcount)+")");
     if(mdp.d_qname != origQname)
-      throw ResolverException(string("resolver: received an answer to another question (")+mdp.d_qname.toString()+"!="+ origQname.toString()+".)");
+      throw ResolverException(string("resolver: received an answer to another question (")+mdp.d_qname.toLogString()+"!="+ origQname.toLogString()+".)");
   }
 
   vector<DNSResourceRecord> ret;
   DNSResourceRecord rr;
-  for(MOADNSParser::answers_t::const_iterator i=mdp.d_answers.begin(); i!=mdp.d_answers.end(); ++i) {
-    rr.qname = i->first.d_name;
-    rr.qtype = i->first.d_type;
-    rr.ttl = i->first.d_ttl;
-    rr.content = i->first.d_content->getZoneRepresentation(true);
+  result->reserve(mdp.d_answers.size());
+
+  for (const auto& i: mdp.d_answers) {
+    rr.qname = i.first.d_name;
+    rr.qtype = i.first.d_type;
+    rr.ttl = i.first.d_ttl;
+    rr.content = i.first.d_content->getZoneRepresentation(true);
     result->push_back(rr);
   }
 
   return 0;
 }
 
-bool Resolver::tryGetSOASerial(DNSName *domain, uint32_t *theirSerial, uint32_t *theirInception, uint32_t *theirExpire, uint16_t* id)
+bool Resolver::tryGetSOASerial(DNSName *domain, ComboAddress* remote, uint32_t *theirSerial, uint32_t *theirInception, uint32_t *theirExpire, uint16_t* id)
 {
   auto fds = std::unique_ptr<struct pollfd[]>(new struct pollfd[locals.size()]);
   size_t i = 0, k;
   int sock;
 
-  for(std::map<string,int>::iterator iter=locals.begin(); iter != locals.end(); iter++, i++) {
-    fds[i].fd = iter->second;
+  for (const auto& iter: locals) {
+    fds[i].fd = iter.second;
     fds[i].events = POLLIN;
+    ++i;
   }
 
   if (poll(fds.get(), i, 250) < 1) { // wait for 0.25s
@@ -243,10 +248,10 @@ bool Resolver::tryGetSOASerial(DNSName *domain, uint32_t *theirSerial, uint32_t
   if (sock < 0) return false; // false alarm
 
   int err;
-  ComboAddress fromaddr;
-  socklen_t addrlen=fromaddr.getSocklen();
+  remote->sin6.sin6_family = AF_INET6; // make sure getSocklen() below returns a large enough value
+  socklen_t addrlen=remote->getSocklen();
   char buf[3000];
-  err = recvfrom(sock, buf, sizeof(buf), 0,(struct sockaddr*)(&fromaddr), &addrlen);
+  err = recvfrom(sock, buf, sizeof(buf), 0,(struct sockaddr*)(remote), &addrlen);
   if(err < 0) {
     if(errno == EAGAIN)
       return false;
@@ -259,25 +264,28 @@ bool Resolver::tryGetSOASerial(DNSName *domain, uint32_t *theirSerial, uint32_t
   *domain = mdp.d_qname;
   
   if(domain->empty())
-    throw ResolverException("SOA query to '" + fromaddr.toStringWithPort() + "' produced response without domain name (RCode: " + RCode::to_s(mdp.d_header.rcode) + ")");
+    throw ResolverException("SOA query to '" + remote->toStringWithPort() + "' produced response without domain name (RCode: " + RCode::to_s(mdp.d_header.rcode) + ")");
 
   if(mdp.d_answers.empty())
-    throw ResolverException("Query to '" + fromaddr.toStringWithPort() + "' for SOA of '" + domain->toString() + "' produced no results (RCode: " + RCode::to_s(mdp.d_header.rcode) + ")");
+    throw ResolverException("Query to '" + remote->toStringWithPort() + "' for SOA of '" + domain->toLogString() + "' produced no results (RCode: " + RCode::to_s(mdp.d_header.rcode) + ")");
   
   if(mdp.d_qtype != QType::SOA)
-    throw ResolverException("Query to '" + fromaddr.toStringWithPort() + "' for SOA of '" + domain->toString() + "' returned wrong record type");
+    throw ResolverException("Query to '" + remote->toStringWithPort() + "' for SOA of '" + domain->toLogString() + "' returned wrong record type");
+
+  if(mdp.d_header.rcode != 0)
+    throw ResolverException("Query to '" + remote->toStringWithPort() + "' for SOA of '" + domain->toLogString() + "' returned Rcode " + RCode::to_s(mdp.d_header.rcode));
 
   *theirInception = *theirExpire = 0;
   bool gotSOA=false;
   for(const MOADNSParser::answers_t::value_type& drc :  mdp.d_answers) {
-    if(drc.first.d_type == QType::SOA) {
+    if(drc.first.d_type == QType::SOA && drc.first.d_name == *domain) {
       shared_ptr<SOARecordContent> src=getRR<SOARecordContent>(drc.first);
       if (src) {
         *theirSerial=src->d_st.serial;
         gotSOA = true;
       }
     }
-    if(drc.first.d_type == QType::RRSIG) {
+    if(drc.first.d_type == QType::RRSIG && drc.first.d_name == *domain) {
       shared_ptr<RRSIGRecordContent> rrc=getRR<RRSIGRecordContent>(drc.first);
       if(rrc && rrc->d_type == QType::SOA) {
         *theirInception= std::max(*theirInception, rrc->d_siginception);
@@ -286,31 +294,15 @@ bool Resolver::tryGetSOASerial(DNSName *domain, uint32_t *theirSerial, uint32_t
     }
   }
   if(!gotSOA)
-    throw ResolverException("Query to '" + fromaddr.toString() + "' for SOA of '" + domain->toString() + "' did not return a SOA");
+    throw ResolverException("Query to '" + remote->toString() + "' for SOA of '" + domain->toLogString() + "' did not return a SOA");
   return true;
 }
 
-int Resolver::resolve(const string &ipport, const DNSName &domain, int type, Resolver::res_t* res, const ComboAddress &local)
+int Resolver::resolve(const ComboAddress& to, const DNSName &domain, int type, Resolver::res_t* res, const ComboAddress &local)
 {
   try {
-    ComboAddress to(ipport, 53);
-
-    int id = sendResolve(to, local, domain, type);
-    int sock;
-
-    // choose socket based on local
-    if (local.sin4.sin_family == 0) {
-      // up to us.
-      sock = to.sin4.sin_family == AF_INET ? locals["default4"] : locals["default6"];
-    } else {
-      std::string lstr = local.toString();
-      std::map<std::string, int>::iterator lptr;
-      // see if there is a local
-
-      if ((lptr = locals.find(lstr)) != locals.end()) sock = lptr->second;
-      else throw ResolverException("sendResolve did not create socket for " + lstr);
-    }
-
+    int sock = -1;
+    int id = sendResolve(to, local, domain, type, &sock);
     int err=waitForData(sock, 0, 3000000); 
   
     if(!err) {
@@ -323,46 +315,50 @@ int Resolver::resolve(const string &ipport, const DNSName &domain, int type, Res
     socklen_t addrlen = sizeof(from);
     char buffer[3000];
     int len;
-    
+
     if((len=recvfrom(sock, buffer, sizeof(buffer), 0,(struct sockaddr*)(&from), &addrlen)) < 0) 
       throw ResolverException("recvfrom error waiting for answer: "+stringerror());
-  
+
+    if (from != to) {
+      throw ResolverException("Got answer from the wrong peer while resolving ("+from.toStringWithPort()+" instead of "+to.toStringWithPort()+", discarding");
+    }
+
     MOADNSParser mdp(false, buffer, len);
     return parseResult(mdp, domain, type, id, res);
   }
   catch(ResolverException &re) {
-    throw ResolverException(re.reason+" from "+ipport);
+    throw ResolverException(re.reason+" from "+to.toLogString());
   }
   return -1;
 }
 
-int Resolver::resolve(const string &ipport, const DNSName &domain, int type, Resolver::res_t* res) {
+int Resolver::resolve(const ComboAddress& ipport, const DNSName &domain, int type, Resolver::res_t* res) {
   ComboAddress local;
   local.sin4.sin_family = 0;
   return resolve(ipport, domain, type, res, local);
 }
 
-void Resolver::getSoaSerial(const string &ipport, const DNSName &domain, uint32_t *serial)
+void Resolver::getSoaSerial(const ComboAddress& ipport, const DNSName &domain, uint32_t *serial)
 {
   vector<DNSResourceRecord> res;
   int ret = resolve(ipport, domain, QType::SOA, &res);
   
   if(ret || res.empty())
-    throw ResolverException("Query to '" + ipport + "' for SOA of '" + domain.toString() + "' produced no answers");
+    throw ResolverException("Query to '" + ipport.toLogString() + "' for SOA of '" + domain.toLogString() + "' produced no answers");
 
   if(res[0].qtype.getCode() != QType::SOA) 
-    throw ResolverException("Query to '" + ipport + "' for SOA of '" + domain.toString() + "' produced a "+res[0].qtype.getName()+" record");
+    throw ResolverException("Query to '" + ipport.toLogString() + "' for SOA of '" + domain.toLogString() + "' produced a "+res[0].qtype.getName()+" record");
 
   vector<string>parts;
   stringtok(parts, res[0].content);
   if(parts.size()<3)
-    throw ResolverException("Query to '" + ipport + "' for SOA of '" + domain.toString() + "' produced an unparseable response");
+    throw ResolverException("Query to '" + ipport.toLogString() + "' for SOA of '" + domain.toLogString() + "' produced an unparseable response");
 
   try {
     *serial=pdns_stou(parts[2]);
   }
   catch(const std::out_of_range& oor) {
-    throw ResolverException("Query to '" + ipport + "' for SOA of '" + domain.toString() + "' produced an unparseable serial");
+    throw ResolverException("Query to '" + ipport.toLogString() + "' for SOA of '" + domain.toLogString() + "' produced an unparseable serial");
   }
 }
 
@@ -370,19 +366,19 @@ AXFRRetriever::AXFRRetriever(const ComboAddress& remote,
                              const DNSName& domain,
                              const TSIGTriplet& tt, 
                              const ComboAddress* laddr,
-                             size_t maxReceivedBytes)
+                             size_t maxReceivedBytes,
+                             uint16_t timeout)
   : d_tsigVerifier(tt, remote, d_trc), d_receivedBytes(0), d_maxReceivedBytes(maxReceivedBytes)
 {
   ComboAddress local;
-  if (laddr != NULL) {
-    local = (ComboAddress) (*laddr);
+  if (laddr != nullptr) {
+    local = ComboAddress(*laddr);
   } else {
-    if(remote.sin4.sin_family == AF_INET)
-      local=ComboAddress(::arg()["query-local-address"]);
-    else if(!::arg()["query-local-address6"].empty())
-      local=ComboAddress(::arg()["query-local-address6"]);
-    else
-      local=ComboAddress("::");
+    string qlas = remote.sin4.sin_family == AF_INET ? "query-local-address" : "query-local-address6";
+    if (::arg()[qlas].empty()) {
+      throw ResolverException("Unable to determine source address for AXFR request to " + remote.toStringWithPort() + " for " + domain.toLogString() + ". " + qlas + " is unset");
+    }
+    local=ComboAddress(::arg()[qlas]);
   }
   d_sock = -1;
   try {
@@ -391,12 +387,12 @@ AXFRRetriever::AXFRRetriever(const ComboAddress& remote,
       throw ResolverException("Error creating socket for AXFR request to "+d_remote.toStringWithPort());
     d_buf = shared_array<char>(new char[65536]);
     d_remote = remote; // mostly for error reporting
-    this->connect();
+    this->connect(timeout);
     d_soacount = 0;
   
     vector<uint8_t> packet;
     DNSPacketWriter pw(packet, domain, QType::AXFR);
-    pw.getHeader()->id = dns_random(0xffff);
+    pw.getHeader()->id = dns_random_uint16();
   
     if(!tt.name.empty()) {
       if (tt.algo == DNSName("hmac-md5"))
@@ -424,7 +420,7 @@ AXFRRetriever::AXFRRetriever(const ComboAddress& remote,
       throw ResolverException("Partial write on AXFR request to "+d_remote.toStringWithPort());
     }
   
-    int res = waitForData(d_sock, 10, 0);
+    int res = waitForData(d_sock, timeout, 0);
     
     if(!res)
       throw ResolverException("Timeout waiting for answer from "+d_remote.toStringWithPort()+" during AXFR");
@@ -446,42 +442,31 @@ AXFRRetriever::~AXFRRetriever()
 
 
 
-int AXFRRetriever::getChunk(Resolver::res_t &res, vector<DNSRecord>* records) // Implementation is making sure RFC2845 4.4 is followed.
+int AXFRRetriever::getChunk(Resolver::res_t &res, vector<DNSRecord>* records, uint16_t timeout) // Implementation is making sure RFC2845 4.4 is followed.
 {
   if(d_soacount > 1)
     return false;
 
   // d_sock is connected and is about to spit out a packet
-  int len=getLength();
+  int len=getLength(timeout);
   if(len<0)
     throw ResolverException("EOF trying to read axfr chunk from remote TCP client");
 
   if (d_maxReceivedBytes > 0 && (d_maxReceivedBytes - d_receivedBytes) < (size_t) len)
     throw ResolverException("Reached the maximum number of received bytes during AXFR");
 
-  timeoutReadn(len);
+  timeoutReadn(len, timeout);
 
   d_receivedBytes += (uint16_t) len;
 
   MOADNSParser mdp(false, d_buf.get(), len);
 
-  int err;
-  if(!records)
-    err=parseResult(mdp, DNSName(), 0, 0, &res);
-  else {
-    records->clear();
-    for(const auto& r: mdp.d_answers)
-      records->push_back(r.first);
-    err = mdp.d_header.rcode;
+  int err = mdp.d_header.rcode;
+
+  if(err) {
+    throw ResolverException("AXFR chunk error: " + RCode::to_s(err));
   }
-  
-  if(err) 
-    throw ResolverException("AXFR chunk error: " + RCode::to_s(err));
 
-  for(const MOADNSParser::answers_t::value_type& answer :  mdp.d_answers)
-    if (answer.first.d_type == QType::SOA)
-      d_soacount++;
- 
   try {
     d_tsigVerifier.check(std::string(d_buf.get(), len), mdp);
   }
@@ -489,16 +474,38 @@ int AXFRRetriever::getChunk(Resolver::res_t &res, vector<DNSRecord>* records) //
     throw ResolverException(re.what());
   }
 
+  if(!records) {
+    err = parseResult(mdp, DNSName(), 0, 0, &res);
+
+    if (!err) {
+      for(const auto& answer :  mdp.d_answers)
+        if (answer.first.d_type == QType::SOA)
+          d_soacount++;
+    }
+  }
+  else {
+    records->clear();
+    records->reserve(mdp.d_answers.size());
+
+    for(auto& r: mdp.d_answers) {
+      if (r.first.d_type == QType::SOA) {
+        d_soacount++;
+      }
+
+      records->push_back(std::move(r.first));
+    }
+  }
+
   return true;
 }
 
-void AXFRRetriever::timeoutReadn(uint16_t bytes)
+void AXFRRetriever::timeoutReadn(uint16_t bytes, uint16_t timeoutsec)
 {
-  time_t start=time(0);
+  time_t start=time(nullptr);
   int n=0;
   int numread;
   while(n<bytes) {
-    int res=waitForData(d_sock, 10-(time(0)-start));
+    int res=waitForData(d_sock, timeoutsec-(time(nullptr)-start));
     if(res<0)
       throw ResolverException("Reading data from remote nameserver over TCP: "+stringerror());
     if(!res)
@@ -513,7 +520,7 @@ void AXFRRetriever::timeoutReadn(uint16_t bytes)
   }
 }
 
-void AXFRRetriever::connect()
+void AXFRRetriever::connect(uint16_t timeout)
 {
   setNonBlocking( d_sock );
 
@@ -534,7 +541,7 @@ void AXFRRetriever::connect()
   if(!err)
     goto done;
 
-  err=waitForRWData(d_sock, false, 10, 0); // wait for writeability
+  err=waitForRWData(d_sock, false, timeout, 0); // wait for writeability
   
   if(!err) {
     try {
@@ -551,7 +558,7 @@ void AXFRRetriever::connect()
     throw ResolverException("Timeout connecting to server");
   }
   else if(err < 0) {
-    throw ResolverException("Error connecting: "+string(strerror(err)));
+    throw ResolverException("Error connecting: "+stringerror());
   }
   else {
     Utility::socklen_t len=sizeof(err);
@@ -567,9 +574,9 @@ void AXFRRetriever::connect()
   // d_sock now connected
 }
 
-int AXFRRetriever::getLength()
+int AXFRRetriever::getLength(uint16_t timeout)
 {
-  timeoutReadn(2);
+  timeoutReadn(2, timeout);
   return (unsigned char)d_buf[0]*256+(unsigned char)d_buf[1];
 }
 
